@@ -36,7 +36,7 @@ from .events import Click, Mount
 from .fuzzy import Matcher
 from .message import Message
 from .reactive import var
-from .screen import Screen, _SystemModalScreen
+from .screen import Screen, SystemModalScreen
 from .timer import Timer
 from .types import CallbackType, IgnoreReturnCallbackType
 from .widget import Widget
@@ -354,7 +354,9 @@ class CommandList(OptionList, can_focus=False):
         border-right: none;
         height: auto;
         max-height: 70vh;
-        background: $panel;
+        background: transparent;
+        padding: 0;
+        text-style: bold;
     }
 
     CommandList:focus {
@@ -370,11 +372,11 @@ class CommandList(OptionList, can_focus=False):
     }
 
     CommandList > .option-list--option-highlighted {
-        background: $accent;
+        background: $primary;
     }
 
     CommandList > .option-list--option {
-        padding-left: 1;
+        padding-left: 2;
     }
     """
 
@@ -410,13 +412,13 @@ class CommandInput(Input):
     CommandInput, CommandInput:focus {
         border: blank;
         width: 1fr;
-        background: $panel;
+        background: transparent;
         padding-left: 0;
     }
     """
 
 
-class CommandPalette(_SystemModalScreen[CallbackType]):
+class CommandPalette(SystemModalScreen[CallbackType]):
     """The Textual command palette."""
 
     COMPONENT_CLASSES: ClassVar[set[str]] = {
@@ -431,18 +433,19 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     """
 
     DEFAULT_CSS = """
+    
+   
     CommandPalette:inline {
         /* If the command palette is invoked in inline mode, we may need additional lines. */
         min-height: 20;
     }
     CommandPalette {
-        background: $background 30%;
-        align-horizontal: center;
+        background: $background 60%;
+        align-horizontal: center;      
     }
 
-    CommandPalette > .command-palette--help-text {
-        background: transparent;
-        color: $text-muted;
+    CommandPalette > .command-palette--help-text {           
+        text-style: dim not bold;       
     }
 
     CommandPalette:dark > .command-palette--highlight {
@@ -455,17 +458,16 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     }
 
     CommandPalette > Vertical {
-        margin-top: 3;
-        width: 90%;
+        margin-top: 3; 
         height: 100%;
         visibility: hidden;
+        background: $primary 20%;      
     }
 
     CommandPalette #--input {
         height: auto;
         visibility: visible;
         border: hkey $primary;
-        background: $panel;
     }
 
     CommandPalette #--input.--list-visible {
@@ -490,7 +492,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     CommandPalette LoadingIndicator {
         height: auto;
         visibility: hidden;
-        background: $panel;
         border-bottom: hkey $primary;
     }
 
@@ -624,14 +625,14 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         with Vertical():
             with Horizontal(id="--input"):
                 yield SearchIcon()
-                yield CommandInput(placeholder="Command Palette Search...")
+                yield CommandInput(placeholder="Search for commands…")
                 if not self.run_on_select:
                     yield Button("\u25b6")
             with Vertical(id="--results"):
                 yield CommandList()
                 yield LoadingIndicator()
 
-    def _on_click(self, event: Click) -> None:
+    def _on_click(self, event: Click) -> None:  # type: ignore[override]
         """Handle the click event.
 
         Args:
@@ -664,7 +665,7 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             provider._post_init()
         self._gather_commands("")
 
-    async def _on_unmount(self) -> None:
+    async def _on_unmount(self) -> None:  # type: ignore[override]
         """Shutdown providers when command palette is closed."""
         if self._providers:
             await wait(
@@ -718,7 +719,7 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
                 command_list = self.query_one(CommandList)
                 command_list.add_option(
                     Option(
-                        Align.center(Text("No matches found")),
+                        Align.center(Text("No matches found", style="not bold")),
                         disabled=True,
                         id=self._NO_MATCHES,
                     )
@@ -853,41 +854,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             for search in searches:
                 search.cancel()
 
-    @staticmethod
-    def _sans_background(style: Style) -> Style:
-        """Returns the given style minus the background color.
-
-        Args:
-            style: The style to remove the color from.
-
-        Returns:
-            The given style, minus its background.
-        """
-        # Here we're pulling out all of the styles *minus* the background.
-        # This should probably turn into a utility method on Style
-        # eventually. The reason for this is we want the developer to be
-        # able to style the help text with a component class, but we want
-        # the background to always be the background at any given moment in
-        # the context of an OptionList. At the moment this act of copying
-        # sans bgcolor seems to be the only way to achieve this.
-        return Style(
-            blink2=style.blink2,
-            blink=style.blink,
-            bold=style.bold,
-            color=style.color,
-            conceal=style.conceal,
-            dim=style.dim,
-            encircle=style.encircle,
-            frame=style.frame,
-            italic=style.italic,
-            link=style.link,
-            overline=style.overline,
-            reverse=style.reverse,
-            strike=style.strike,
-            underline2=style.underline2,
-            underline=style.underline,
-        )
-
     def _refresh_command_list(
         self, command_list: CommandList, commands: list[Command], clear_current: bool
     ) -> None:
@@ -934,8 +900,8 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
 
         # We'll potentially use the help text style a lot so let's grab it
         # the once for use in the loop further down.
-        help_style = self._sans_background(
-            self.get_component_rich_style("command-palette--help-text")
+        help_style = self.get_component_rich_style(
+            "command-palette--help-text", partial=True
         )
 
         # The list to hold on to the commands we've gathered from the
@@ -996,7 +962,9 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             # list of commands that have been gathered so far.
             prompt = hit.prompt
             if hit.help:
-                prompt = Group(prompt, Text(hit.help, style=help_style))
+                help_text = Text(hit.help)
+                help_text.stylize(help_style)
+                prompt = Group(prompt, help_text)
             gathered_commands.append(Command(prompt, hit, id=str(command_id)))
 
             # Before we go making any changes to the UI, we do a quick
