@@ -1,32 +1,27 @@
-"""Provides the core of a classic vertical bounce-bar option list.
-
-Useful as a lightweight list view (not to be confused with ListView, which
-is much richer but uses widgets for the items) and as the base for various
-forms of bounce-bar menu.
-"""
-
 from __future__ import annotations
 
-from typing import ClassVar, Iterable, NamedTuple
+from typing import TYPE_CHECKING, ClassVar, Iterable, NamedTuple
 
+import rich.repr
 from rich.console import RenderableType
 from rich.measure import Measurement
 from rich.padding import Padding
-from rich.repr import Result
 from rich.rule import Rule
-from rich.style import Style
-from typing_extensions import Self, TypeAlias
+from rich.style import NULL_STYLE, Style
 
-from .. import _widget_navigation
+from .. import _widget_navigation, events
 from .._widget_navigation import Direction
 from ..binding import Binding, BindingType
-from ..events import Click, Idle, Leave, MouseMove
+from ..cache import LRUCache
 from ..geometry import Region, Size
 from ..message import Message
 from ..reactive import reactive
 from ..scroll_view import ScrollView
 from ..strip import Strip
 
+if TYPE_CHECKING:
+    from typing_extensions import Self
+
 
 class DuplicateID(Exception):
     """Raised if a duplicate ID is used when adding options to an option list."""
@@ -36,6 +31,11 @@ class OptionDoesNotExist(Exception):
     """Raised when a request has been made for an option that doesn't exist."""
 
 
+class Separator:
+    """Class used to add a separator to an [OptionList][textual.widgets.OptionList]."""
+
+
+@rich.repr.auto
 class Option:
     """Class that holds the details of an individual option."""
 
@@ -71,27 +71,13 @@ class Option:
         """The optional ID for the option."""
         return self.__id
 
-    def __rich_repr__(self) -> Result:
+    def __rich_repr__(self) -> rich.repr.Result:
         yield "prompt", self.prompt
         yield "id", self.id, None
         yield "disabled", self.disabled, False
 
-
-class Separator:
-    """Class used to add a separator to an [OptionList][textual.widgets.OptionList]."""
-
-
-class Line(NamedTuple):
-    """Class that holds a list of segments for the line of a option."""
-
-    segments: Strip
-    """The strip of segments that make up the line."""
-
-    option_index: int | None = None
-    """The index of the [Option][textual.widgets.option_list.Option] that this line is related to.
-
-    If the line isn't related to an option this will be `None`.
-    """
+    def __rich__(self) -> RenderableType:
+        return self.__prompt
 
 
 class OptionLineSpan(NamedTuple):
@@ -109,14 +95,6 @@ class OptionLineSpan(NamedTuple):
     line_count: int
     """The count of lines that make up the option."""
 
-    def __contains__(self, line: object) -> bool:
-        # For this named tuple `in` will have a very specific meaning; but
-        # to keep mypy and friends happy we need to accept an object as the
-        # parameter. So, let's keep the type checkers happy but only accept
-        # an int.
-        assert isinstance(line, int)
-        return line >= self.first and line < (self.first + self.line_count)
-
 
 OptionListContent: TypeAlias = "Option | Separator"
 """The type of an item of content in the option list.
@@ -157,24 +135,6 @@ class OptionList(ScrollView, can_focus=True):
     | up | Move the highlight up. |
     """
 
-    COMPONENT_CLASSES: ClassVar[set[str]] = {
-        "option-list--option",
-        "option-list--option-disabled",
-        "option-list--option-highlighted",
-        "option-list--option-hover",
-        "option-list--option-hover-highlighted",
-        "option-list--separator",
-    }
-    """
-    | Class | Description |
-    | :- | :- |
-    | `option-list--option-disabled` | Target disabled options. |
-    | `option-list--option-highlighted` | Target the highlighted option. |
-    | `option-list--option-hover` | Target an option that has the mouse over it. |
-    | `option-list--option-hover-highlighted` | Target a highlighted option that has the mouse over it. |
-    | `option-list--separator` | Target the separators. |
-    """
-
     DEFAULT_CSS = """
     OptionList {
         height: auto;
@@ -225,6 +185,15 @@ class OptionList(ScrollView, can_focus=True):
     }
     """
 
+    COMPONENT_CLASSES: ClassVar[set[str]] = {
+        "option-list--option",
+        "option-list--option-disabled",
+        "option-list--option-highlighted",
+        "option-list--option-hover",
+        "option-list--option-hover-highlighted",
+        "option-list--separator",
+    }
+
     highlighted: reactive[int | None] = reactive["int | None"](None)
     """The index of the currently-highlighted option, or `None` if no option is highlighted."""
 
@@ -257,7 +226,7 @@ class OptionList(ScrollView, can_focus=True):
             """
             return self.option_list
 
-        def __rich_repr__(self) -> Result:
+        def __rich_repr__(self) -> rich.repr.Result:
             yield "option_list", self.option_list
             yield "option", self.option
             yield "option_id", self.option_id
@@ -287,23 +256,8 @@ class OptionList(ScrollView, can_focus=True):
         wrap: bool = True,
         tooltip: RenderableType | None = None,
     ):
-        """Initialise the option list.
-
-        Args:
-            *content: The content for the option list.
-            name: The name of the option list.
-            id: The ID of the option list in the DOM.
-            classes: The CSS classes of the option list.
-            disabled: Whether the option list is disabled or not.
-            wrap: Should prompts be auto-wrapped?
-            tooltip: Optional tooltip.
-        """
         super().__init__(name=name, id=id, classes=classes, disabled=disabled)
 
-        # Internal refresh trackers. For things driven from on_idle.
-        self._needs_refresh_content_tracking = False
-        self._needs_to_scroll_to_highlight = False
-
         self._wrap = wrap
         """Should we auto-wrap options?
 
@@ -333,39 +287,99 @@ class OptionList(ScrollView, can_focus=True):
         """
 
         self._option_ids: dict[str, int] = {
-            option.id: index for index, option in enumerate(self._options) if option.id
+            option.id: index
+            for index, option in enumerate(self._options)
+            if option.id is not None
         }
         """A dictionary of option IDs and the option indexes they relate to."""
 
-        self._lines: list[Line] = []
-        """A list of all of the individual lines that make up the option list.
+        self._content_render_cache: LRUCache[tuple[int, Style, int], list[Strip]]
+        self._content_render_cache = LRUCache(256)
 
-        Note that the size of this list will be at least the same as the number
-        of options, and actually greater if any prompt of any option is
-        multiple lines.
-        """
-
-        self._spans: list[OptionLineSpan] = []
-        """A list of the locations and sizes of all options in the option list.
-
-        This will be the same size as the number of prompts; each entry in
-        the list contains the line offset of the start of the prompt, and
-        the count of the lines in the prompt.
-        """
-
-        # Initial calculation of the content tracking.
-        self._request_content_tracking_refresh()
+        self._lines: list[tuple[int, int]] | None = None
+        self._spans: list[OptionLineSpan] | None = None
 
         self._mouse_hovering_over: int | None = None
         """Used to track what the mouse is hovering over."""
 
-        # Finally, cause the highlighted property to settle down based on
-        # the state of the option list in regard to its available options.
-        self.action_first()
-
         if tooltip is not None:
             self.tooltip = tooltip
 
+        if self._options:
+            self.action_first()
+
+    def _left_gutter_width(self) -> int:
+        """Returns the size of any left gutter that should be taken into account.
+
+        Returns:
+            The width of the left gutter.
+        """
+        return 0
+
+    def _on_mount(self):
+        self._populate()
+
+    def _refresh_lines(self) -> None:
+        self._lines = None
+        self._spans = None
+        self._content_render_cache.clear()
+        self.check_idle()
+
+    def notify_style_update(self) -> None:
+        self._content_render_cache.clear()
+
+    def _on_resize(self):
+        self._refresh_lines()
+
+    def on_idle(self):
+        if self._lines is None:
+            self._populate()
+
+    def _add_lines(
+        self, new_content: list[OptionListContent], width: int, option_index=0
+    ) -> None:
+        """Add new lines.
+
+        Args:
+            new_content: New content to add.
+            width: Width to render content.
+            option_index: Starting option index.
+        """
+        assert self._lines is not None
+        assert self._spans is not None
+        style = NULL_STYLE
+
+        for index, content in enumerate(new_content, len(self._lines)):
+            if isinstance(content, Option):
+                height = len(
+                    self._render_option_content(
+                        index, content, style, width - self._left_gutter_width()
+                    )
+                )
+
+                self._lines.extend(
+                    (option_index, y_offset) for y_offset in range(height)
+                )
+                self._spans.append(OptionLineSpan(option_index, height))
+                option_index += 1
+            else:
+                self._lines.append(OptionLineSpan(-1, 0))
+
+        self.virtual_size = Size(width, len(self._lines))
+
+    def _populate(self) -> None:
+        """Populate the lines data-structure."""
+        if self._lines is not None:
+            return
+        self._lines = []
+        self._spans = []
+
+        self._add_lines(
+            self._contents,
+            self.scrollable_content_region.width - self._left_gutter_width(),
+        )
+        self.refresh()
+
     def get_content_width(self, container: Size, viewport: Size) -> int:
         """Get maximum width of options."""
         console = self.app.console
@@ -375,43 +389,7 @@ class OptionList(ScrollView, can_focus=True):
             for option in self._options
         )
 
-    def _request_content_tracking_refresh(
-        self, rescroll_to_highlight: bool = False
-    ) -> None:
-        """Request that the content tracking information gets refreshed.
-
-        Args:
-            rescroll_to_highlight: Should the widget ensure the highlight is visible?
-
-        Calling this method sets a flag to say the refresh should happen,
-        and books the refresh call in for the next idle moment.
-        """
-        self._needs_refresh_content_tracking = True
-        self._needs_to_scroll_to_highlight = rescroll_to_highlight
-        self.check_idle()
-
-    async def _on_idle(self, _: Idle) -> None:
-        """Perform content tracking data refresh when idle."""
-        self._refresh_content_tracking()
-        if self._needs_to_scroll_to_highlight:
-            self._needs_to_scroll_to_highlight = False
-            self.scroll_to_highlight()
-
-    def watch_show_vertical_scrollbar(self) -> None:
-        """Handle the vertical scrollbar visibility status changing.
-
-        `show_vertical_scrollbar` is watched because it has an impact on the
-        available width in which to render the renderables that make up the
-        options in the list. If a vertical scrollbar appears or disappears
-        we need to recalculate all the lines that make up the list.
-        """
-        self._request_content_tracking_refresh()
-
-    def _on_resize(self) -> None:
-        """Refresh the layout of the renderables in the list when resized."""
-        self._request_content_tracking_refresh(rescroll_to_highlight=True)
-
-    def _on_mouse_move(self, event: MouseMove) -> None:
+    def _on_mouse_move(self, event: events.MouseMove) -> None:
         """React to the mouse moving.
 
         Args:
@@ -419,11 +397,11 @@ class OptionList(ScrollView, can_focus=True):
         """
         self._mouse_hovering_over = event.style.meta.get("option")
 
-    def _on_leave(self, _: Leave) -> None:
+    def _on_leave(self, _: events.Leave) -> None:
         """React to the mouse leaving the widget."""
         self._mouse_hovering_over = None
 
-    async def _on_click(self, event: Click) -> None:
+    async def _on_click(self, event: events.Click) -> None:
         """React to the mouse being clicked on an item.
 
         Args:
@@ -449,97 +427,37 @@ class OptionList(ScrollView, can_focus=True):
             return Separator()
         return Option(content)
 
-    def _clear_content_tracking(self) -> None:
-        """Clear down the content tracking information."""
-        self._lines.clear()
-        self._spans.clear()
-
-    def _left_gutter_width(self) -> int:
-        """Returns the size of any left gutter that should be taken into account.
-
-        Returns:
-            The width of the left gutter.
-        """
-        return 0
-
-    def _refresh_content_tracking(self, force: bool = False) -> None:
-        """Refresh the various forms of option list content tracking.
+    def _render_option_content(
+        self, option_index: int, renderable: RenderableType, style: Style, width: int
+    ) -> list[Strip]:
+        """Render content for option and style.
 
         Args:
-            force: Optionally force the refresh.
+            option_index: Option index to render.
+            renderable: The Option renderable.
+            style: The Rich style to render with.
+            width: The width of the renderable.
 
-        Raises:
-            DuplicateID: If there is an attempt to use a duplicate ID.
-
-        Without a `force` the refresh will only take place if it has been
-        requested via `_refresh_content_tracking`.
+        Returns:
+            A list of strips.
         """
+        cache_key = (option_index, style, width)
+        if (strips := self._content_render_cache.get(cache_key, None)) is not None:
+            return strips
 
-        # If we don't need to refresh, don't bother.
-        if not self._needs_refresh_content_tracking and not force:
-            return
-
-        # If we don't know our own width yet, we can't sensibly work out the
-        # heights of the prompts of the options yet, so let's shortcut that
-        # work. We'll be back here once we know our height.
-        if not self.size.width:
-            return
-
-        self._clear_content_tracking()
-        self._needs_refresh_content_tracking = False
-
-        # Set up for doing less property access work inside the loop.
-        lines_from = self.app.console.render_lines
-        add_span = self._spans.append
-        add_lines = self._lines.extend
-
-        # Adjust the options for our purposes.
-        options = self.app.console.options.update_width(
-            self.scrollable_content_region.width - self._left_gutter_width()
-        )
-        options.no_wrap = not self._wrap
-        if not self._wrap:
-            options.overflow = "ellipsis"
-
-        # Create a rule that can be used as a separator.
-        separator = Strip(lines_from(Rule(style=""))[0])
-
-        # Work through each item that makes up the content of the list,
-        # break out the individual lines that will be used to draw it, and
-        # also set up the tracking of the actual options.
-        line = 0
-        option_index = 0
         padding = self.get_component_styles("option-list--option").padding
-        for content in self._contents:
-            if isinstance(content, Option):
-                # The content is an option, so render out the prompt and
-                # work out the lines needed to show it.
-                new_lines = [
-                    Line(
-                        Strip(prompt_line).apply_style(
-                            Style(meta={"option": option_index})
-                        ),
-                        option_index,
-                    )
-                    for prompt_line in lines_from(
-                        Padding(content.prompt, padding) if padding else content.prompt,
-                        options,
-                    )
-                ]
-                # Record the span information for the option.
-                add_span(OptionLineSpan(line, len(new_lines)))
-                option_index += 1
-            else:
-                # The content isn't an option, so it must be a separator (if
-                # there were to be other non-option content for an option
-                # list it's in this if/else where we'd process it).
-                new_lines = [Line(separator)]
-            add_lines(new_lines)
-            line += len(new_lines)
+        console = self.app.console
+        options = console.options.update_width(width)
+        if not self._wrap:
+            options = options.update(no_wrap=True, overflow="ellipsis")
+        if padding:
+            renderable = Padding(renderable, padding)
+        lines = self.app.console.render_lines(renderable, options, style=style)
 
-        # Now that we know how many lines make up the whole content of the
-        # list, set the virtual size.
-        self.virtual_size = Size(self.scrollable_content_region.width, len(self._lines))
+        style_meta = Style.from_meta({"option": option_index})
+        strips = [Strip(line, width).apply_style(style_meta) for line in lines]
+        self._content_render_cache[cache_key] = strips
+        return strips
 
     def _duplicate_id_check(self, candidate_items: list[OptionListContent]) -> None:
         """Check the items to be added for any duplicates.
@@ -584,9 +502,15 @@ class OptionList(ScrollView, can_focus=True):
         """
         # Only work if we have items to add; but don't make a fuss out of
         # zero items to add, just carry on like nothing happened.
-        if items:
+        if self._lines is None:
+            self._lines = []
+        if self._spans is None:
+            self._spans = []
+        new_items = list(items)
+        if new_items:
+            option_index = len(self._options)
             # Turn any incoming values into valid content for the list.
-            content = [self._make_content(item) for item in items]
+            content = [self._make_content(item) for item in new_items]
             self._duplicate_id_check(content)
             self._contents.extend(content)
             # Pull out the content that is genuine options, create any new
@@ -600,7 +524,11 @@ class OptionList(ScrollView, can_focus=True):
                     self._option_ids[new_option.id] = new_option_index
             self._options.extend(new_options)
 
-            self._refresh_content_tracking(force=True)
+            self._add_lines(
+                content,
+                self.scrollable_content_region.width - self._left_gutter_width(),
+                option_index=option_index,
+            )
             self.refresh()
         return self
 
@@ -636,11 +564,10 @@ class OptionList(ScrollView, can_focus=True):
             for option_id, option_index in self._option_ids.items()
             if option_index != index
         }
-        self._refresh_content_tracking(force=True)
+        self._refresh_lines()
         # Force a re-validation of the highlight.
         self.highlighted = self.highlighted
         self._mouse_hovering_over = None
-        self.refresh()
 
     def remove_option(self, option_id: str) -> Self:
         """Remove the option with the given ID.
@@ -688,8 +615,7 @@ class OptionList(ScrollView, can_focus=True):
             OptionDoesNotExist: If there is no option with the given index.
         """
         self.get_option_at_index(index).set_prompt(prompt)
-        self._refresh_content_tracking(force=True)
-        self.refresh()
+        self._refresh_lines()
 
     def replace_option_prompt(self, option_id: str, prompt: RenderableType) -> Self:
         """Replace the prompt of the option with the given ID.
@@ -736,8 +662,7 @@ class OptionList(ScrollView, can_focus=True):
         self._option_ids.clear()
         self.highlighted = None
         self._mouse_hovering_over = None
-        self.virtual_size = Size(self.scrollable_content_region.width, 0)
-        self._refresh_content_tracking(force=True)
+        self._refresh_lines()
         return self
 
     def _set_option_disabled(self, index: int, disabled: bool) -> Self:
@@ -877,80 +802,59 @@ class OptionList(ScrollView, can_focus=True):
             ) from None
 
     def render_line(self, y: int) -> Strip:
-        """Render a single line in the option list.
+        self._populate()
+        assert self._lines is not None
 
-        Args:
-            y: The Y offset of the line to render.
-
-        Returns:
-            A `Strip` instance for the caller to render.
-        """
-
-        scroll_x, scroll_y = self.scroll_offset
-
-        # First off, work out which line we're working on, based off the
-        # current scroll offset plus the line we're being asked to render.
+        _scroll_x, scroll_y = self.scroll_offset
         line_number = scroll_y + y
+
         try:
-            line = self._lines[line_number]
+            option_index, y_offset = self._lines[line_number]
         except IndexError:
-            # An IndexError means we're drawing in an option list where
-            # there's more list than there are options.
             return Strip([])
 
-        # Now that we know which line we're on, pull out the option index so
-        # we have a "local" copy to refer to rather than needing to do a
-        # property access multiple times.
-        option_index = line.option_index
+        renderable = (
+            Rule(style=self.get_component_rich_style("option-list--separator"))
+            if option_index == -1
+            else self._options[option_index]
+        )
 
-        # Knowing which line we're going to be drawing, we can now go pull
-        # the relevant segments for the line of that particular prompt.
-        strip = line.segments
+        mouse_over = self._mouse_hovering_over == option_index
 
-        # If the line we're looking at isn't associated with an option, it
-        # will be a separator, so let's exit early with that.
-        if option_index is None:
-            return strip.apply_style(
-                self.get_component_rich_style("option-list--separator")
-            )
+        component_class: str | None = None
 
-        # At this point we know we're drawing actual content. To allow for
-        # horizontal scrolling, let's crop the strip at the right locations.
-        strip = strip.crop(scroll_x, scroll_x + self.scrollable_content_region.width)
+        if option_index == -1:
+            component_class = "option-list--separator"
+        else:
+            try:
+                option = self._options[option_index]
+            except IndexError:
+                pass
+            else:
+                if option.disabled:
+                    component_class = "option-list--option-disabled"
+                elif self.highlighted == option_index:
+                    component_class = "option-list--option-highlighted"
+                elif mouse_over:
+                    component_class = "option-list--option-hover"
 
-        highlighted = self.highlighted
-        mouse_over = self._mouse_hovering_over
-        spans = self._spans
+        style = (
+            self.get_component_rich_style(component_class)
+            if component_class
+            else self.rich_style
+        )
 
-        # Handle drawing a disabled option.
-        if self._options[option_index].disabled:
-            return strip.apply_style(
-                self.get_component_rich_style("option-list--option-disabled")
-            )
-
-        # Handle drawing a highlighted option.
-        if highlighted is not None and line_number in spans[highlighted]:
-            # Highlighted with the mouse over it?
-            if option_index == mouse_over:
-                return strip.apply_style(
-                    self.get_component_rich_style(
-                        "option-list--option-hover-highlighted"
-                    )
-                )
-            # Just a normal highlight.
-            return strip.apply_style(
-                self.get_component_rich_style("option-list--option-highlighted")
-            )
-
-        # Perhaps the line is within an otherwise-uninteresting option that
-        # has the mouse hovering over it?
-        if mouse_over is not None and line_number in spans[mouse_over]:
-            return strip.apply_style(
-                self.get_component_rich_style("option-list--option-hover")
-            )
-
-        # It's a normal option line.
-        return strip.apply_style(self.rich_style)
+        strips = self._render_option_content(
+            option_index,
+            renderable,
+            style,
+            self.scrollable_content_region.width - self._left_gutter_width(),
+        )
+        try:
+            strip = strips[y_offset]
+        except IndexError:
+            return Strip([])
+        return strip
 
     def scroll_to_highlight(self, top: bool = False) -> None:
         """Ensure that the highlighted option is in view.
@@ -959,19 +863,18 @@ class OptionList(ScrollView, can_focus=True):
             top: Scroll highlight to top of the list.
         """
         highlighted = self.highlighted
-        if highlighted is None:
+        if highlighted is None or self._spans is None:
             return
+
         try:
-            span = self._spans[highlighted]
+            y, height = self._spans[highlighted]
         except IndexError:
             # Index error means we're being asked to scroll to a highlight
             # before all the tracking information has been worked out.
             # That's fine; let's just NoP that.
             return
         self.scroll_to_region(
-            Region(
-                0, span.first, self.scrollable_content_region.width, span.line_count
-            ),
+            Region(0, y, self.scrollable_content_region.width, height),
             force=True,
             animate=False,
             top=top,
@@ -1034,6 +937,10 @@ class OptionList(ScrollView, can_focus=True):
         # If we find ourselves in a position where we don't know where we're
         # going, we need a fallback location. Where we go will depend on the
         # direction.
+        self._populate()
+        assert self._spans is not None
+        assert self._lines is not None
+
         fallback = self.action_first if direction == -1 else self.action_last
 
         highlighted = self.highlighted
@@ -1052,7 +959,7 @@ class OptionList(ScrollView, can_focus=True):
             try:
                 # Now that we've got a target line, let's figure out the
                 # index of the target option.
-                target_option = self._lines[target_line].option_index
+                target_option: int | None = self._lines[target_line][0]
             except IndexError:
                 # An index error suggests we've gone out of bounds, let's
                 # settle on whatever the call thinks is a good place to wrap
@@ -1094,3 +1001,14 @@ class OptionList(ScrollView, can_focus=True):
         highlighted = self.highlighted
         if highlighted is not None and not self._options[highlighted].disabled:
             self.post_message(self.OptionSelected(self, highlighted))
+
+
+if __name__ == "__main__":
+    from textual.app import App, ComposeResult
+
+    class OptionApp(App):
+        def compose(self) -> ComposeResult:
+            yield OptionList("Foo", "Bar", "Baz")
+
+    app = OptionApp()
+    app.run()
